unit u_main;

{  DISCLAIMER
This software is free to use, copy or distrubute as a whole or in parts, as you like.
I, the author do not guarantee proper operation, suitability for any use,
  correctness of results, etc.....
I do not take any responsibility for misbehaviour of the software,
  including affecting the system on which it runs.
I can give some support regarding the operation of the program.
I cannot give any support for problems related to Free Pascal or Lazarus.
If you have such, visit the FPC and Lazarus forum(s).
This software was developed and tested on
  - WXP and W7 with FPC version 2.6.2 and Lazarus version 1.0.14.
  - Linux Mint with FPC version 2.6.4 and Lazarus version 1.4.2.

This software requires the packet Sdpo-Serial to be installed in the IDE.

January 2019 jan@breem.nl
}

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics,
  Dialogs, ExtCtrls, StdCtrls, SdpoSerial, StrUtils;

type

  { TFrmMain }

  TFrmMain = class(TForm)
    CbxCrydom: TCheckBox;
    CbxTalk: TCheckBox;
    CbxRST: TCheckBox;
    Label10: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    ButtClrClips: TLabel;
    Label19: TLabel;
    Label9: TLabel;
    LblSupplyFail: TLabel;
    LblRstTPA: TLabel;
    LblMainsFail: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label16: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    LblFault: TLabel;
    Lbl50: TLabel;
    Lbl12: TLabel;
    Lbl5A: TLabel;
    Lblmin5A: TLabel;
    LblOTW: TLabel;
    Serial: TSdpoSerial;
    TbxV50: TEdit;
    TbxV12: TEdit;
    TbxV5A: TEdit;
    TbxInMessageLength: TEdit;
    TbxSerialPortName: TEdit;
    TbxV5D: TEdit;
    TbxClips: TEdit;
    TbxVMin5A: TEdit;
    TbxInMessage: TEdit;
    TbxOutCount: TEdit;
    TbxInCount: TEdit;
    TbxOutMessage: TEdit;
    TbxAdcRef: TEdit;
    TbxOS: TEdit;
    Timer: TTimer;
    procedure ButtClrClipsClick(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure CbxTalkChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end;

var
  FrmMain: TFrmMain;

implementation
{$R *.lfm}

uses u_numstrings;

const
  VERSIONSTRING = '2019-01-22';
  INMESSAGESIZE = 13;

var
  InMessage, OutMessage, SerialPortName, SettingsFileName: shortstring;
  Status, Command, Clips, InCount, OutCount: byte;
  V5D, V50, V12, V5A, VMin5A: real;
  AdcRef: integer;

procedure DecodeInMessage;
begin
  with FrmMain do
  begin
  // Decompose message
  Status:= byte(InMessage[1]);
  AdcRef:= StrLittleToUnSignedInt(Midstr(InMessage,2,2)); // ADC value
  V50:= StrLittleToUnSignedInt(Midstr(InMessage,4,2));    // millivolts
  V12:= StrLittleToUnSignedInt(Midstr(InMessage,6,2));    // millivolts
  V5A:= StrLittleToUnSignedInt(Midstr(InMessage,8,2));    // millivolts
  Vmin5A:= StrLittleToSignedInt(Midstr(InMessage,10,2));  // millivolts
  Clips:= byte(InMessage[12]); // The testTool does not use clips MT
  // Process supply voltages
  TbxAdcRef.text:= format('%3d',[AdcRef]);
  if AdcRef > 400 then  // prevent division by zero.
  V5D:= 2.5 * 1024.0 / AdcRef else V5D:= 0;
  TbxV50.text:= format('%4.1f',[V50 / 1000]);
  TbxV12.text:= format('%4.1f',[V12 / 1000]);
  TbxV5A.text:= format('%4.1f',[V5A / 1000]);
  TbxVMin5A.text:= format('%4.1f',[Vmin5A / 1000]);
  TbxV5D.text:= format('%4.1f',[V5D]);
  // Range check uses the same margins as in the firmware
  if(V50 < 42000) or (V50 > 55000) then Lbl50.Color:= clRed else Lbl50.Color:= clLime;
	if(V12 < 10000) or (V12 > 13000) then Lbl12.Color:= clRed else Lbl12.Color:= clLime;
	if(V5A < 4500) or (V5A > 5500) then Lbl5A.Color:= clRed else Lbl5A.Color:= clLime;
	if(Vmin5A < -5500) or (Vmin5A > -4500) then Lblmin5A.Color:= clRed else Lblmin5A.Color:= clLime;
  // Process status byte
  if(Status and $01) = $01 then LblMainsFail.Color:= clRed else LblMainsFail.Color:= clLime;
  if(Status and $02) = $02 then LblSupplyFail.Color:= clRed else LblSupplyFail.Color:= clLime;
  if(Status and $04) = $04 then LblFault.Color:= clRed else LblFault.Color:= clLime;
  if(Status and $10) = $10 then LblOTW.Color:= clRed else LblOTW.Color:= clLime;
  if(Status and $80) = $80 then LblRstTPA.Color:= clRed else LblRstTPA.Color:= clLime;
  if Clips > 0 then ButtClrClips.color:= clRed else ButtClrClips.color:= clLime;
  TbxClips.Text:= format('%3d', [Clips]);
	end;
end;

procedure PrepareOutMessage;
begin
  with FrmMain do
  begin
    if CbxRST.checked then Command:= Command or $80;
    if CbxCrydom.checked then Command:= Command or $10;
    // FLT firmware expects Command, Bass, Mid. So we use 2 dummies.
    OutMessage:= char(Command) + char(192) + char(193);
    Command:= 0;
  end;
end;

procedure TFrmMain.ButtClrClipsClick(Sender: TObject);
begin
  Command += $60;
end;

// This program is not able to receive spontaneous messages from FLT 
procedure TFrmMain.TimerTimer(Sender: TObject);
var InMessageLength: integer;
begin
  InMessage:= Serial.ReadData;
  InMessageLength:= length(InMessage);
  TbxInMessage.text:= BytesView(InMessage,InMessageLength);
  if InMessageLength > 0 then
  begin
    inc(InCount);
    TbxInCount.text:= format('%3d',[InCount]);
  end;
  TbxInMessageLength.text:= format('%3d',[InMessageLength]);
  if InMessageLength = INMESSAGESIZE then DecodeInMessage;
  PrepareOutMessage;
  TbxOutMessage.Text:= BytesView(OutMessage, length(OutMessage));
  inc(OutCount);
  TbxOutCount.text:= format('%3d',[OutCount]);
  Serial.WriteData(OutMessage);
end;

procedure TFrmMain.CbxTalkChange(Sender: TObject);
begin
  if CbxTalk.Checked then
  begin
    Try
      Serial.close;
      SerialPortName:= TbxSerialPortName.text;
      Serial.Device:= SerialPortName;
      Serial.BaudRate:= br_19200;
      Serial.Open;
      Timer.enabled:= true;
    except
      on E: Exception do ShowMessage(E.message);
    end;
  end
  else
  begin
    Timer.enabled := false;
    Serial.Close;
  end;
end;

procedure WriteSettings;
var SettingsFile: text;
begin
  assign(SettingsFile, SettingsFileName);
  rewrite(SettingsFile);
  SerialPortName:= FrmMain.TbxSerialPortName.text;
  writeln(SettingsFile, SerialPortName);
  close(SettingsFile);
end;

procedure ReadSettings;
var SettingsFile: text;
begin
  if not FileExists(SettingsFileName) then exit;
  assign(SettingsFile, SettingsFileName);
  reset(SettingsFile);
  readln(SettingsFile, SerialPortName);
  close(SettingsFile);
  FrmMain.TbxSerialPortName.text:= SerialPortName;
end;

procedure TFrmMain.FormDestroy(Sender: TObject);
begin
  WriteSettings;
end;

procedure TFrmMain.FormCreate(Sender: TObject);
var
  OSstring, ExeDir: shortstring;
begin
{$IFDEF Linux}
  OSstring:= 'Linux';
{$ENDIF}
{$IFDEF WINDOWS}
  OSstring:= 'Windows';
{$ENDIF}
// todo: MAC
  TbxOS.text:= OSstring;
  Caption:= 'TestTool TPA v: ' + VERSIONSTRING;
  ExeDir := application.Location; // directory of executable
  SettingsFilename:= ExeDir + 'Settings.txt';
  ReadSettings;
end;

end.
